; Fichier de macro-commandes pour assembleur 80x86
; macconv.inc : macro-commandes de conversion ascii, binaire, hexa, decimal.
; Bertrand Ducourthial. Fevrier 1998. Version 2.

; ********** Macro de conversion d'un binaire en chaine ascii **********
; Entree   : arg 1 : nombre de bits.
;            arg 2 : valeur binaire.
;            arg 3 : adresse de la chaine destination.
; Action   : Conversion de arg 1 bits (en commancant par la droite) de arg 2
;            en caracteres ascii sauvegardes dans la chaine arg 3.
; Retour   : Chaine de arg 1 caracteres terminee par $, a l'adresse arg 3.
; Remarque : La chaine pointee par arg 3 doit etre de taille arg 1 + 1.
;            Si arg 1 est plus grand que le nombre de bits de la valeur
;            binaire, des 0 sont ajoutes a gauche dans la chaine resultat.
;            arg 1 doit etre plus grand que 0.
;            Les parametres arg 1 et arg 2 doivent etre des registres ou des
;            adresses en memoire.

convertir_bin_ascii MACRO _nbits, _binaire, _adresse
          LOCAL _boucle

          ; Sauvegarde des registres utilises dans la macro.
          PUSH    AX
          PUSH    BX
          PUSH    CX
          PUSH    SI

          ; Recuperation des parametres.
          PUSH    _nbits    ; Prevention de l'ecrasement de parametres.
          PUSH    _binaire  ; Ex. : convertir_bin_ascii CX, AX, chaine.
          MOV     BX, OFFSET _adresse
          POP     AX        ; AX = _binaire
          POP     CX        ; CX = _nbits

          MOV     SI, CX
          MOV     BYTE PTR [BX+SI], '$' ; Caractere de fin de chaine.

 _boucle: SHR     AX, 1     ; Decalage d'un bit vers la droite et recuperation
                            ; du bit de poids faible dans CF
          MOV     SI, CX    ; SI - 1 = numero du caractere courant.
          MOV     BYTE PTR [BX+SI-1], '0' ; Ce caractere est '0' + valeur du
          ADC     BYTE PTR [BX+SI-1], 0   ; bit qui s'est retrouve dans CF.
          LOOP    _boucle   ; CX est decremente pour le caractere suivant.

          ; Restauration des registres empiles.
          POP     SI
          POP     CX
          POP     BX
          POP     AX
        ENDM


; ********** Macro de conversion d'un hexadecimal en chaine ascii **********
; Entree   : arg 1 : nombre de chiffres hexadecimaux a convertir.
;            arg 2 : valeur hexadecimale.
;            arg 3 : adresse de la chaine destination.
; Action   : Conversion de arg 1 chiffres hexadecimaux (en commancant par la
;            droite) de arg 2 en caracteres ascii sauvegardes dans la chaine
;            arg 3.
; Retour   : Chaine de arg 1 caracteres terminee par $, a l'adresse arg 3.
; Remarque : La chaine pointee par arg 3 doit etre de taille arg 1 + 1.
;            Si arg 1 est plus grand que le nombre de chiffres de la valeur
;            hexadecimale, des 0 sont ajoutes a gauche dans la chaine resultat.
;            arg 1 doit etre plus grand que 0.
;            Les parametres arg 1 et arg 2 doivent etre des registres ou des
;            adresses en memoire.

convertir_hex_ascii MACRO _nchiffres, _hexa, _adresse
          LOCAL _boucle, _suite, _lettre

          ; Sauvegarde des registres utilises dans la macro.
          PUSH    AX
          PUSH    BX
          PUSH    CX
          PUSH    DX
          PUSH    SI

          ; Recuperation des parametres.
          PUSH    _nchiffres ; Prevention de l'ecrasement de parametres.
          PUSH    _hexa      ; Ex. : convertir_hex_ascii AX, CX, chaine.
          MOV     BX, OFFSET _adresse
          POP     AX         ; AX = _hexa
          POP     CX         ; CX = _nchiffres

          MOV     SI, CX
          MOV     BYTE PTR [BX+SI], '$' ; Caractere de fin de chaine.

 _boucle: MOV     DX, AX
          AND     DX, 0Fh    ; Masque tous les bits sauf les 4 de poids faible
                             ; du nombre hexadecimal courant.
          CMP     DX, 0Ah    ; Comparaison de DX et 10.
          JGE     _lettre    ; Si DX >= 10, il faut ecrire une lettre.
                             ; Sinon, il faut ecrire un chiffre :
          ADD     DL, '0'    ; DL = code ascii du chiffre.
          JMP     _suite

 _lettre: ADD     DL, 'A'-10 ; DL = code ascii de la lettre.

 _suite:
          PUSH    CX         ; Sauvegarde de CX pour l'utiliser
          MOV     CL, 4      ; temporairement au decalage de AX.
          SHR     AX, CL     ; Decalage de 4 bits vers la droite de AX.
          POP     CX         ; Restauration de CX

          MOV     SI, CX
          MOV     BYTE PTR [BX+SI-1], DL ; Copie du caractere lettre
                                         ; ou chiffre dans la chaine.
          LOOP    _boucle    ; CX est decremente pour le caractere suivant.

          ; Restauration des registres empiles.
          POP     SI
          POP     DX
          POP     CX
          POP     BX
          POP     AX
        ENDM


; ********** Macro de conversion d'un decimal en chaine ascii **********
; Entree   : arg 1 : nombre de chiffres decimaux a convertir
;            arg 2 : valeur decimale
;            arg 3 : adresse de la chaine destination
; Action   : Conversion de arg 1 chiffres decimaux (en commancant par la
;            droite) de arg 2 en caracteres ascii sauvegardes dans la chaine
;            arg 3.
; Retour   : Chaine de arg 1 caracteres terminee par $, a l'adresse arg 3
; Remarque : La chaine pointee par arg 3 doit etre de taille arg 1 + 1.
;            Si arg 1 est plus grand que le nombre de chiffres de la valeur
;            decimale, des 0 sont ajoutes a gauche dans la chaine resultat.
;            arg 1 doit etre plus grand que 0.
;            arg 1 et arg 2 doivent etre des registres ou des adresses memoires.
;            Telle qu'est faite la division, elle implique que la valeur
;            decimale a convertir doit etre inferieure strictement a 256.
;            Cette macro necessite l'inclusion des fichiers macinit.inc et
;            macaffi.inc.

convertir_dec_ascii MACRO _nchiffres, _dec, _adresse
          LOCAL _boucle1, _boucle2, _fin, _suite

          ; Sauvegarde des registres utilises dans la macro.
          PUSH    AX
          PUSH    BX
          PUSH    CX
          PUSH    DX
          PUSH    SI

          ; Recuperation des parametres
          PUSH    _nchiffres ; Prevention de l'ecrasement de parametres.
          PUSH    _dec       ; Ex. : convertir_dec_ascii AX, CX, chaine.
          MOV     BX, OFFSET _adresse
          POP     AX         ; AX = _dec
          POP     CX         ; CX = _nchiffres

          CMP     AX, 0FFh   ; Comparaison du decimal avec la valeur max
                             ; autorisee par la division sur 8 bits.
          JBE     _suite     ; Le decimal est < 256, donc ok.

                             ; Le decimal est > 255, donc probleme.
          PUSH    DS         ; Sauvegarde de l'adresse du registre de segment
          MOV     AX, DONNEES_MACRO ; initial pour en utiliser un autre.
          MOV     DS, AX
          afficher_texte erreur_conv_dec_ascii ; affichage du message d'erreur.
          POP     DS         ; Restauration de l'adresse du segment de donnees.
                             ; Construction d'une chaine avec la valeur max.
          MOV     SI, CX
          MOV     BYTE PTR [BX+SI], '$'  ; Caractere de fin de chaine.
          MOV     BYTE PTR [BX+SI-1], '5'
          MOV     BYTE PTR [BX+SI-2], '5'
          MOV     BYTE PTR [BX+SI-3], '2'
          SUB     CX, 3      ; On complete le reste de la chaine par des 0.

_boucle1: MOV     SI, CX
          MOV     BYTE PTR [BX+SI-1], '0'
          LOOP    _boucle1               ; CX est decremente. Fin si CX=0.
          JMP     _fin                   ; Fin de la macro.

  _suite: MOV     SI, CX
          MOV     BYTE PTR [BX+SI], '$'  ; Caractere de fin de chaine.

          MOV     DL, 0Ah                ; Pour la division par 10.

_boucle2: DIV     DL                     ; Division de AX par DL=10 :
                                         ;  AH = reste, AL = quotient.
          ADD     AH, '0'                ; Caractere ascii du chiffre.

          MOV     SI, CX                 ; Copie du caractere dans la
          MOV     BYTE PTR [BX+SI-1], AH ; chaine.
                                         
          AND     AX, 00FFh              ; On ne garde que le quotient.
          LOOP    _boucle2               ; CX est decremente. Fin si CX=0.


          ; Restauration des registres empiles.
    _fin: POP     SI
          POP     DX
          POP     CX
          POP     BX
          POP     AX
        ENDM


; ********** Macro de calcul de la longueur d'une chaine ascii **********
; Entree   : arg 1 : adresse de la chaine ascii.
; Action   : Calcul du nombre de caracteres ascii precedant le caractere
;            '$' de fin de chaine.
; Retour   : SI = nombre de caracteres ascii dans la chaine pointee par
;            arg 1.
; Remarque : La chaine doit effectivement se terminer par un tel caractere.

calculer_longueur_chaine MACRO _adresse
          LOCAL _alpha, _beta

          ; Sauvegarde des registres utilises dans la macro.
          PUSH    BX

          ; Recuperation des parametres
          MOV     BX, OFFSET _adresse

          MOV     SI, 0
 _alpha:  CMP     BYTE PTR[BX+SI], '$'
          JE      _beta
          INC     SI
          JMP     _alpha

 _beta:    ; Restauration des registres empiles.
          POP     BX
        ENDM


; ********** Macro de conversion d'une chaine ascii en nombre **********
; Entree   : arg 1 : adresse de la chaine a convertir
;            arg 2 : base dans laquelle est exprime le nombre contenu dans
;                    la chaine.
; Action   : Conversion du nombre contenu dans la chaine pointee par arg 1
;            et exprime en base arg 2 en un nombre sauvegarde dans AX.
; Retour   : Nombre dans le registre AX.
; Remarque : arg 2 doit etre un registre ou une adresse memoire. La valeur de
;            retour est limitee a la contenance max de AX, soit 65535. Si le
;            nombre a convertir est plus grand, un message d'erreur est affiche
;            et le resultat est 65535.
;            Cette macro necessite l'inclusion des fichiers macinit.inc et
;            macaffi.inc.

convertir_ascii_nombre MACRO _adresse, _base
          LOCAL _boucle, _erreur, _ajout, _fin, _suite, _suite2

          ; Sauvegarde des registres utilises dans la macro.
          PUSH    BX
          PUSH    CX
          PUSH    DX
          PUSH    SI
          PUSH    BP

          ; Recuperation des parametres
          PUSH    _base          ; Sauvegarde de _base sur la pile.
          MOV     BX, OFFSET _adresse

          calculer_longueur_chaine _adresse ; SI = longueur de la chaine.

          MOV     AX, 0          ; AX = decimal courant.
          MOV     CX, SI         ; CX = nombre de boucles a faire.
          MOV     DX, 1          ; Base courante pour le premier caractere.

 _boucle: MOV     SI, CX         ; SI=num du caractere courant dans la chaine.
          MOV     CX, AX         ; AX contient la valeur courante du decimal.
                                 ; Mais AX va servir (MUL). On utilise CX dont
                                 ; la valeur est sauvegardee par SI pour sauve-
                                 ; garder temporairement la valeur de AX.
          MOV     AH, 0
          MOV     AL, BYTE PTR [BX+SI-1]  ; Recuperation du caractere ascii.
          SUB     AL, '0'        ; Transformation en chiffre.
          CMP     AL, 9
          JBE     _suite
          SUB     AL, 7
  _suite: PUSH    DX             ; Sauvegarde de DX modifie par MUL DX.
          MUL     DX             ; Multiplication AX <- AX x DX.

          CMP     DX, 0          ; Test des poids forts du resultat de la
          JA      _erreur        ; multiplication. Si non nul, probleme car
                                 ; cela implique que le nombre a convertir
                                 ; est superieur a 65535.

                                 ; Dans le cas ou le nombre a ajouter au decimal
                                 ; n'est pas superieur a 65535, il faut encore
                                 ; verifier que le decimal courant ne va pas
                                 ; depasser 65535 apres l'addition.
          MOV     DX, 0FFFFh     ; DX <- valeur maxi d'un registre 16 bits.
          SUB     DX, CX         ; DX <- FFFFh - decimal courant.
          CMP     DX, AX         ; Comparaison de DX et de AX, valeur que l'on
                                 ; veut ajouter au decimal courant.
          JAE     _ajout         ; Ok, on peut encore augmenter le decimal
                                 ; courant.

                                 ; On ne peut plus augmenter le decimal
                                 ; courant. Message d'erreur et fin.
 _erreur: POP     DX             ; Il faut depiler autant qu'on a empile.
          PUSH    DS             ; Sauvegarde de l'adresse du registre de
          MOV     AX, DONNEES_MACRO ; segment de donnees pour utiliser un autre
          MOV     DS, AX         ; segment de donnees (cf. macinit.inc).
          afficher_texte erreur_conv_ascii_nombre ; Message d'erreur.
          POP     DS
          MOV     AX, 0FFFFh     ; Le resultat est au maximum.
          JMP     _fin           ; Fin de la macro.


  _ajout: POP     DX             ; Restauration de DX.
          ADD     AX, CX         ; Ajout de l'ancienne valeur du decimal.
                                 ; De nouveau, AX = valeur courante du decimal.
          MOV     CX, SI         ; Restauration de la valeur de CX.

          ; Multiplication de la base courante DX par _base pour
          ; traiter le caractere suivant.
          PUSH    AX             ; Sauvegarde de AX.
          MOV     AX, DX         ; Utilisation de AX pour DX <- DX x _base.
          MOV     BP, SP         ; Necessaire pour l'indirection ci-dessous.
          MOV     DX, WORD PTR [BP+2] ; DX = _base sauvegarde sur la pile avant
                                      ; le PUSH AX.
          MUL     DX             ; Multiplication AX <- AX x DX.

 _suite2: MOV     DX, AX         ; Finalement, DX <- DX x _base.
          POP     AX             ; Restauration de AX.

          LOOP    _boucle        ; CX est decremente. Fin si CX=0.

    _fin: POP     BX             ; Depilement de _base.

          ; Restauration des registres empiles.
          POP     BP
          POP     SI
          POP     DX
          POP     CX
          POP     BX

        ENDM

