; Fichier de macro-commandes pour assembleur 80x86
; macfich.inc : macro-commandes de gestion des fichiers.
; Bertrand Ducourthial fevrier 1998. Version 2.

; ********** Macro d'ouverture d'un fichier texte **********
; Entree   : arg 1 : mode d'ouverture du fichier.
;            arg 2 : nom du fichier.
;            arg 3 : action en cas d'erreur (nombre 0 ou 1).
; Action   : Ouverture du fichier de nom arg 2 avec les droits specifies par
;            arg 1 :
;                   arg 1 = 0 -> le fichier est ouvert en lecture seule.
;                   arg 1 = 1 -> le fichier est ouvert en ecriture seule.
;                   arg 1 = 2 -> le fichier est ouvert en lecture et ecriture.
;            En cas de probleme d'ouverture du fichier, affichage d'un message
;            d'erreur. De plus, si arg 3 est different de 0, arret du
;            programme (si arg 3 = 0, fin normale de la macro).
; Retour   : AX = numero permettant de manipuler le fichier ulterieurement
;            (handle).
; Remarque : arg 1 doit etre une chaine de caractere terminee par $. Elle peut
;            contenir tout un chemin (C:\USER\ALFRED\toto) ou un nom simple,
;            mais pas de jockers (? et *). Le fichier doit deja exister.
;            arg 1 et arg 3 doivent etre des valeurs directes. arg 2 doit etre
;            une adresse en memoire.
;            Cette macro necessite la macro l'inclusion des fichiers
;            macinit.inc, macaffi.inc et macconv.inc.

ouvrir_fichier MACRO  _mode, _ad_nom, _action_erreur
          LOCAL _fin

          ; Sauvegarde des registres utilises dans la macro.
          PUSH    BX
          PUSH    CX
          PUSH    DX
          PUSH    SI

          ; Recuperation des parametres
          calculer_longueur_chaine _ad_nom
          MOV     [_ad_nom+SI], 0    ; Le nom doit se terminer par 0.
          LEA     DX, _ad_nom
          MOV     AL, _mode
          MOV     CX, _action_erreur

          ; Interruption d'ouverture de fichier.
          MOV     AH, 3Dh
          INT     21h

          MOV     [_ad_nom+SI], '$'  ; On remet un '$' a la fin du nom.

          ; Test du bon deroulement de l'ouverture du fichier.
          JNC     _fin

          ; Traitement de l'erreur :
          PUSH    DS             ; Sauvegarde de l'adresse du registre de
          MOV     AX, DONNEES_MACRO ; segment de donnees pour utiliser un autre
          MOV     DS, AX         ; segment de donnees (cf. macinit.inc).
          afficher_texte erreur_ouverture_fichier ; Affichage du message
          afficher_texte _ad_nom ; et du nom de fichier qui a pose probleme.
          POP     DS

          CMP     CX, 0     ; Test de l'action a realiser en cas d'erreur.
          JE      _fin      ; _action_erreur = 0 => fin normale de la macro.
          MOV     AX, 4C00h ; _action_erreur != 0 => arret du programme.
          INT     21h

          ; Restauration des registres empiles.
    _fin: POP     SI
          POP     DX
          POP     CX
          POP     BX
         ENDM


; ********** Macro de fermeture d'un fichier texte **********
; Entree   : arg 1 : numero du fichier (handle) fourni a l'ouverture.
; Action   : Fermeture du fichier de numero arg 1 (ecriture des donnees
;            restantes a ecrire).
; Retour   : Aucun.
; Remarque :

fermer_fichier MACRO   _num
          ; Sauvegarde des registres utilises dans la macro.
          PUSH    BX

          ; Recuperation des parametres
          MOV     BX, _num

          ; Interruption de fermeture de fichier.
          MOV     AH, 3Eh
          INT     21h

          ; Restauration des registres empiles.
          POP     BX
        ENDM


; ********** Macro de lecture d'un fichier **********
; Entree   : arg 1 : numero du fichier (handle) fourni a l'ouverture.
;            arg 2 : nombre d'octets a lire.
;            arg 3 : adresse de la zone de destination (buffer)
;            arg 4 : adresse par rapport au debut du segment de la zone de
;                    destination (adresse d'offset du buffer)
; Action   : Lecture de arg 2 octets dans le fichier de numero arg 1. Ces
;            octets sont places en memoire a l'adresse (arg 3):(arg 4). Ils
;            sont lus dans le fichier a partir de la position courante du
;            pointeur de fichier.
; Retour   : Aucun.
; Remarque : Les donnees de la zone en memoire sont ecrasees par celles lues
;            dans le fichier. Apres lecture, le pointeur de fichier pointe
;            sur le prochain octet a lire.
;            arg 1 et arg 2 doivent etre des adresses memoires ou des registres.

lire_fichier MACRO   _num, _nb_octets, _adresse_tampon
          ; Sauvegarde des registres utilises dans la macro.
          PUSH    AX
          PUSH    BX
          PUSH    CX
          PUSH    DS
          PUSH    DX

          ; Recuperation des parametres
          PUSH    _num         ; Prevention de l'ecrasement de parametres.
          PUSH    _nb_octets   ; Ex. : lire_fichier DX, BX, CX
          LEA     DX, _adresse_tampon
          POP     CX           ; CX = _nb_octets
          POP     BX           ; BX = _num

          ; Interruption de lecture de fichier.
          MOV     AH, 3Fh
          INT     21h

          ; Restauration des registres empiles.
          POP     DX
          POP     DS
          POP     CX
          POP     BX
          POP     AX
        ENDM


; ********** Macro d'ecriture dans un fichier **********
; Entree   : arg 1 : numero du fichier (handle).
;            arg 2 : nombre d'octets a ecrire.
;            arg 3 : adresse des octets en memoire.
; Action   : Ecriture de arg 2 octets situes en memoire a l'adresse arg 1
;            dans le fichier de numero arg 1 (handle).
; Retour   : Aucun.
; Remarque : arg 1 et arg 2 doivent etre des registres ou des adresses memoire.

ecrire_fichier MACRO   _num, _nb_octets, _adresse_tampon
          ; Sauvegarde des registres utilises dans la macro.
          PUSH    AX
          PUSH    BX
          PUSH    CX
          PUSH    DX

          ; Recuperation des parametres
          PUSH    _num         ; Prevention de l'ecrasement de parametres.
          PUSH    _nb_octets   ; Ex. : ecrire_fichier DX, BX, CX
          LEA     DX, _adresse_tampon ; Adresse ou se trouvent ces octets.
          POP     CX           ; CX = _nb_octets (Nb d'octets a ecrire).
          POP     BX           ; BX = _num (handle du fichier).

          ; Interruption d'ecriture dans un fichier.
          MOV     AH, 40h
          INT     21h

          ; Restauration des registres empiles.
          POP     DX
          POP     CX
          POP     BX
          POP     AX
        ENDM


; ********** Macro de deplacement du pointeur de fichier **********
; Entree   : arg 1 : numero du fichier (handle).
;          : arg 2 : valeur du deplacement.
; Action   : Deplacement de arg 2 octets du pointeur de fichier de numero
;            arg 1 par rapport au debut du fichier.
; Retour   : Aucun.
; Remarque : La valeur du deplacement doit tenir sur 16 bits (arg 2 < 65536).
;            arg 1 doit etre une adresse memoire ou un registre.

deplacer_pointeur_fichier MACRO   _num, _nb_octets
          ; Sauvegarde des registres utilises dans la macro.
          PUSH    AX
          PUSH    BX
          PUSH    CX
          PUSH    DX

          ; Recuperation des parametres
          PUSH    _num           ; Prevention de l'ecrasement de parametres.
          MOV     DX, _nb_octets
          POP     BX
          MOV     CX, 0          ; Deplacement CX:DX sur 16 bits seulement.

          ; Interruption de deplacement d'un pointeur de fichier.
          MOV     AH, 42h
          MOV     AL, 0     ; Distance par rapport au debut du fichier.
          INT     21h

          ; Restauration des registres empiles.
          POP     DX
          POP     CX
          POP     BX
          POP     AX
        ENDM


; ********** Macro de calcul de la taille d'un fichier **********
; Entree   : arg 1 : numero du fichier (handle).
; Action   : Calcul de la taille d'un fichier.
; Retour   : Taille du fichier dans DX:AX (DX poids forts, AX poids faibles)
; Remarque : Le pointeur de fichier se retrouve au debut du fichier apres
;            l'execution de la macro.

calculer_taille_fichier MACRO   _num
          ; Sauvegarde des registres utilises dans la macro.
          PUSH    BX
          PUSH    CX

          ; Recuperation des parametres
          MOV     BX, _num
          MOV     CX, 0
          MOV     DX, 0

          ; Interruption de deplacement d'un pointeur de fichier.
          MOV     AH, 42h
          MOV     AL, 2     ; Distance par rapport a la fin du fichier.
          INT     21h       ; Le pointeur est a la fin. Il est egal a la
                            ; taille du fichier. Il est sauvegarde dans
                            ; DX:AX (DX poids forts et AX poids faibles)

          ; Le pointeur de fichier se retrouve a la fin.
          MOV     CX, 0
          deplacer_pointeur_fichier BX, CX  ; On le remets au debut.

          ; Restauration des registres empiles.
          POP     CX
          POP     BX
        ENDM

