; Fichier de macro-commandes pour assembleur 80x86
; macsais.inc : macro-commandes de saisie de caracteres.
; Bertrand Ducourthial. Fevrier 1998. Version 2.


; ********** Macro de saisie d'un texte (chaine de caracteres) **********
; Entree   : arg 1 : nombre de caracteres a saisir.
;            arg 2 : adresse d'une zone memoire.
; Action   : Saisie d'une chaine d'au plus arg 1 caracteres, stockee en
;            memoire a l'adresse arg 2. La chaine est completee par '$',
;            le caractere de fin de chaine.
; Retour   : Chaine de caracteres a l'adresse arg 2 en memoire.
; Remarque : La zone memoire doit etre libre et de taille arg 1  +  3
;            octets. Les 2 derniers octets sont inutilises apres le retour.
;            Le nombre de caracteres a saisir doit etre inferieur a 65534.
;            Le 2eme argument ne doit pas etre le registre AX. En redirigeant
;            l'entree standard, il est possible de saisir des caracteres en
;            provenance d'un autre peripherique.

saisir_texte MACRO   _nbcar, _adresse
          LOCAL _boucle, _fin   ; Label unique pour chaque "appel" de la macro.

          ; Sauvegarde des registres utilises dans la macro.
          PUSH    AX
          PUSH    BX
          PUSH    CX
          PUSH    DX
          PUSH    SI

          ; Recuperation des parametres.
          MOV     AH, _nbcar    
          LEA     DX, _adresse

          MOV     BX, DX        ; BX est utilise pour l'adressage de base.
          MOV     SI, 0         ; et SI pour l'adressage indexe.

          ADD     AH, 1         ; Il faut aussi saisir le retour chariot.

          ; Interruption de saisie d'une chaine de caracteres.
          ; (avec remise a zero du buffer clavier)
          MOV     [BX], AH
          MOV     AH, 0Ch
          MOV     AL, 0Ah
          INT     21h
          
          MOV     CL, [BX + 1]
          MOV     CH, 0         ; CX = nombre de caracteres saisis.

          CMP     CX, 0  ; Si CX = 0, pas de boucle a faire (aucun caractere
          JE      _fin   ; saisi). Ce test permet d'eviter une boucle infinie.

          ; Decalage de 2 vers la gauche des octets de la chaine.
 _boucle: MOV     AH, [BX+SI+2]
          MOV     [BX + SI], AH
          INC     SI
          LOOP    _boucle   ; Autant de boucles que de caracteres saisis (CX).

          ; Ajout du caractere '$' (fin de chaine).
    _fin: MOV     BYTE PTR [BX+SI], '$'

          ; Restauration des registres empiles.
          POP     SI
          POP     DX
          POP     CX
          POP     BX
          POP     AX

        ENDM


; ********** Macro de saisie d'un caractere **********
; Entree   : arg 1 : registre 8 bits.
; Action   : Saisie d'un seul caractere au clavier.
; Retour   : Le code ascii du caractere saisi est stocke dans arg 1.
; Remarque : arg 1 doit ne doit pas etre le registre AX.

saisir_caractere MACRO   _registre
          ; Sauvegarde des registres utilises dans la macro.
          PUSH    AX

          ; Interruption de saisie d'une chaine de caracteres.
          ; (avec remise a zero du buffer clavier)
          MOV     AH, 01h
          MOV     AL, 0Ch
          INT     21h

          ; Sauvegarde du resultat dans le registre _registre.
          MOV     _registre, AL

          ; Restauration du registre empile.
          POP     AX

        ENDM

; ********** Macro de saisie d'un nombre hexadecimal dans AX **********
; Entree   :
; Action   : Saisie d'un nombre hexadecimal dans le registre AX.
; Retour   : Nombre dans le registre AX.
; Remarque : Le nombre hexadecimal est compris entre 0 et FFFF.
;            Si d'autres caracteres que 0..9 et A..F sont saisis,
;            un message d'erreur apparait.

saisir_nombre_AX MACRO
          LOCAL _erreur, _boucle, _fin_b, _fin_m
          ; Labels uniques pour chaque "appel" de la macro.

          ; Sauvegarde des registres utilises dans la macro.
          PUSH    BX
          PUSH    CX
          PUSH    DX
          PUSH    DS
          PUSH    SI

          MOV     AX, DONNEES_MACRO     ; Utilisation du segment de donnees
          MOV     DS, AX                ; des macros (cf. macinit.inc) dans
                                        ; toute cette macro.

          afficher_texte saisir_nombre_invite_saisie  ; Affichage d'un message.

          ; Interruption de saisie d'une chaine de caracteres.
          ; (avec remise a zero du buffer clavier)
          LEA     DX, saisir_nombre_chaine

          MOV     BX, DX           ; BX est utilise pour l'adressage base.
          MOV     SI, 0            ; SI est utilise pour l'adressage indexe.

          MOV     BYTE PTR [BX], 5 ; 4 chiffres a saisir + retour chariot.
          MOV     AH, 0Ch
          MOV     AL, 0Ah
          INT     21h
          
          MOV     CL, [BX + 1]
          MOV     CH, 0            ; CX = nombre de caracteres saisis.


          CMP     CX, 0            ; Si CX = 0, chaine vide => erreur.
          JNE     _boucle          ; Sinon, examen des 4 caracteres (boucle).

 _erreur:
          afficher_texte erreur_saisir_nombre ; Message d'erreur.
          MOV   AX, 0                         ; AX = 0 si erreur de saisie.
          MOV   BX, offset _fin_m             ; Fin de la macro. Saut a longue
          JMP   BX                            ; distance.

 _boucle: MOV   AH, BYTE PTR [BX+SI+2]        ; Recuperation du caractere.

          CMP   AH, '0'                       ; Erreur s'il est < '0'.
          JB    _erreur
          CMP   AH, 'F'                       ; Erreur s'il est > 'F'.
          JG    _erreur

          CMP   AH, '9'                       ; Ok s'il est <= '9'.
          JBE   _fin_b
          CMP   AH, 'A'                       ; Ok s'il est >= 'A'.
          JAE   _fin_b
          JMP   _erreur                       ; Sinon; erreur.

  _fin_b: MOV   BYTE PTR [BX+SI], AH
          INC   SI                            ; SI <- SI + 1 pour le caractere
                                              ; suivant.
          LOOP _boucle                        ; On recommence la boucle autant
                                              ; de fois qu'il y a de caracteres
                                              ; saisis (CX).

          MOV   BYTE PTR [BX+SI], '$'         ; Caractere de fin de chaine
                                              ; necessaire pour la macro de
                                              ; conversion.

          MOV   CX, 16                        ; Conversion en hexadecimal (base
          convertir_ascii_nombre saisir_nombre_chaine, CX ; 16 de la chaine
                                              ; ASCII saisir_nombre_saisie.
                                              ; Resultat dans AX.


          ; Restauration des registres empiles.
  _fin_m: POP     SI
          POP     DS
          POP     DX
          POP     CX
          POP     BX

        ENDM

